@@ -23,7 +23,7 @@
  * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
  * SUCH DAMAGE.
  *
- * $Id: bbs_thread_view.c,v 1.46.2.1 2004/05/04 13:52:30 fuyu Exp $
+ * $Id: bbs_thread_view.c,v 1.46.2.5 2004/06/11 12:17:14 fuyu Exp $
  */
 
 #include "config.h"
@@ -60,13 +60,8 @@ static void bbs_thread_view_class_init(BBSThreadViewClass *klass);
 static void bbs_thread_view_init(BBSThreadView *view);
 static void bbs_thread_view_finalize(GObject *object);
 static void bbs_thread_view_destroy(GtkObject *object);
+static void bbs_thread_view_realize(GtkWidget *widget);
 
-static void text_view_realize_cb(GtkTextView *text_view, BBSThreadView *view);
-
-static gboolean text_view_get_iter_at_link_tag_from_event(
-						GtkTextView *text_view,
-						GtkTextIter *iter,
-						GdkEventAny *event);
 static gboolean bbs_thread_view_motion_notify(GtkWidget *widget,
 					      GdkEventMotion *event);
 static gboolean bbs_thread_view_leave_notify(GtkWidget *widget,
@@ -75,10 +70,12 @@ static gboolean bbs_thread_view_button_press(GtkWidget *widget,
 					     GdkEventButton *event);
 static gboolean bbs_thread_view_button_release(GtkWidget *widget,
 					       GdkEventButton *event);
-static void emit_link_mouse_over(BBSThreadView *view, GdkEventMotion *event);
+static void emit_link_mouse_over(BBSThreadView *view, GdkEventMotion *event,
+				 const gchar *link);
 static void emit_link_mouse_out(BBSThreadView *view, GdkEventAny *event);
 
-static void bbs_thread_view_scroll_to(BBSThreadView *view, int to_where);
+static void bbs_thread_view_scroll_to(BBSThreadView *view,
+				      BBSThreadViewScrollTo where);
 
 
 GType
@@ -109,6 +106,26 @@ bbs_thread_view_get_type(void)
 }
 
 
+GType
+bbs_thread_view_scroll_to_get_type(void)
+{
+  static GType scroll_to_type = 0;
+  if (scroll_to_type == 0)
+    {
+      static const GEnumValue values[] =
+	{
+	  { SCROLL_TO_BUFFER_START, (char *)"SCROLL_TO_BUFFER_START",
+	    (char *)"scroll-to-buffer-start" },
+	  { SCROLL_TO_BUFFER_END, (char *)"SCROLL_TO_BUFFER_END",
+	    (char *)"scroll-to-buffer-end" },
+	  { 0, NULL, NULL },
+	};
+      scroll_to_type = g_enum_register_static("BBSThreadViewScrollTo", values);
+    }
+  return scroll_to_type;
+}
+
+
 enum {
   LINK_MOUSE_OVER_SIGNAL,
   LINK_MOUSE_OUT_SIGNAL,
@@ -123,15 +140,9 @@ enum {
 };
 
 
-enum {
-  SCROLL_TO_BUFFER_START,
-  SCROLL_TO_BUFFER_END
-};
-
-
 static GtkTextViewClass *parent_class = NULL;
 static int bbs_thread_view_signals[LAST_SIGNAL] =
-  { 0, 0, 0, 0, 0, 0, 0, 0, 0, };
+  { 0, 0, 0, 0, 0, 0, 0, 0, 0 };
 static GtkTextTagTable *default_tag_table;
 
 static GtkTextTag *link_tags[2];
@@ -435,6 +446,7 @@ bbs_thread_view_class_init(BBSThreadViewClass *klass)
   gtk_binding_entry_add_signal(binding_set, GDK_W, GDK_CONTROL_MASK,
 			       "close_thread", 0);
 
+  widget_class->realize = bbs_thread_view_realize;
   widget_class->motion_notify_event = bbs_thread_view_motion_notify;
   widget_class->leave_notify_event = bbs_thread_view_leave_notify;
   widget_class->button_press_event = bbs_thread_view_button_press;
@@ -464,19 +476,15 @@ trace_free(gpointer pointer)
 static void
 bbs_thread_view_init(BBSThreadView *view)
 {
-  GtkWidget *widget = GTK_WIDGET(view);
   GtkTextBuffer *buffer = gtk_text_buffer_new(default_tag_table);
 
-  gtk_text_view_set_buffer(GTK_TEXT_VIEW(view), buffer);
+  gtk_text_view_set_buffer(&view->text_view, buffer);
 
   view->text_buffer = buffer;
 
-  gtk_text_view_set_wrap_mode(GTK_TEXT_VIEW(view), GTK_WRAP_CHAR);
-  gtk_text_view_set_editable(GTK_TEXT_VIEW(view), FALSE);
-  gtk_text_view_set_cursor_visible(GTK_TEXT_VIEW(view), FALSE);
-
-  g_signal_connect(G_OBJECT(widget), "realize",
-		   G_CALLBACK(text_view_realize_cb), view);
+  gtk_text_view_set_wrap_mode(&view->text_view, GTK_WRAP_CHAR);
+  gtk_text_view_set_editable(&view->text_view, FALSE);
+  gtk_text_view_set_cursor_visible(&view->text_view, FALSE);
 
   gtk_text_buffer_get_start_iter(buffer, &view->iter);
 
@@ -575,16 +583,25 @@ bbs_thread_view_destroy(GtkObject *object)
 
 
 static void
-text_view_realize_cb(GtkTextView *text_view, BBSThreadView *view)
+bbs_thread_view_realize(GtkWidget *widget)
 {
-  GdkDisplay *display = gtk_widget_get_display(GTK_WIDGET(text_view));
-  GdkWindow *window = gtk_text_view_get_window(text_view,
-					       GTK_TEXT_WINDOW_TEXT);
+  BBSThreadView *view;
+  GdkDisplay *display;
+  GdkWindow *window;
+
+  if (GTK_WIDGET_CLASS(parent_class)->realize)
+    (*GTK_WIDGET_CLASS(parent_class)->realize)(widget);
+
+  window = gtk_text_view_get_window(GTK_TEXT_VIEW(widget),
+				    GTK_TEXT_WINDOW_TEXT);
   gdk_window_set_events(window,
 			(gdk_window_get_events(window)
 			 & ~GDK_POINTER_MOTION_HINT_MASK)
 			| GDK_ENTER_NOTIFY_MASK | GDK_LEAVE_NOTIFY_MASK);
 
+  display = gtk_widget_get_display(widget);
+  view = BBS_THREAD_VIEW(widget);
+
   if (view->ibeam_cursor != NULL)
     gdk_cursor_unref(view->ibeam_cursor);
   view->ibeam_cursor = gdk_cursor_new_for_display(display, GDK_XTERM);
@@ -633,13 +650,8 @@ text_view_get_iter_from_event(GtkTextView *text_view, GtkTextIter *iter,
 
 
 static gboolean
-text_view_get_iter_at_link_tag_from_event(GtkTextView *text_view,
-					  GtkTextIter *iter,
-					  GdkEventAny *event)
+text_view_move_iter_at_link_tag(GtkTextView *text_view, GtkTextIter *iter)
 {
-  if (!text_view_get_iter_from_event(text_view, iter, event))
-    return FALSE;
-
   if (gtk_text_iter_has_tag(iter, link_tags[0]))
     {	/* link_tags[0]ǰϤޤƤϰϤؤƤ */
       if (gtk_text_iter_begins_tag(iter, link_tags[0]))
@@ -693,125 +705,10 @@ text_view_get_iter_at_link_tag_from_event(GtkTextView *text_view,
 }
 
 
-static gchar *
-bbs_thread_view_get_selected_text_under_pointer(BBSThreadView *view,
-						GtkTextIter *start,
-						GdkEventAny *event)
-{
-  GtkTextIter end;
-  GtkTextIter iter;
-
-  if (!gtk_text_buffer_get_selection_bounds(view->text_buffer,
-					    start, &end))
-    return NULL;	/* 򤵤Ƥʤ */
-
-  if (!text_view_get_iter_from_event(&view->text_view, &iter, event))
-    return NULL;
-
-  if (!gtk_text_iter_in_range(&iter, start, &end))
-    return NULL;
-
-  return gtk_text_iter_get_visible_text(start, &end);
-}
-
-
-static gboolean
-bbs_thread_view_motion_notify(GtkWidget *widget, GdkEventMotion *event)
-{
-  gboolean result = FALSE;
-  GtkTextIter iter;
-  GtkTextView *text_view;
-  BBSThreadView *view;
-  gchar *text;
-
-  g_return_val_if_fail(IS_BBS_THREAD_VIEW(widget)
-		       && event->type == GDK_MOTION_NOTIFY, FALSE);
-
-  text_view = GTK_TEXT_VIEW(widget);
-  view = BBS_THREAD_VIEW(widget);
-
-  text = bbs_thread_view_get_selected_text_under_pointer(view,
-							 &iter,
-							 (GdkEventAny *)event);
-  if (text != NULL)
-    {
-      int start_offset = gtk_text_iter_get_offset(&iter);
-      if (start_offset != view->last_offset)
-	{
-	  if (view->last_offset != -1)
-	    emit_link_mouse_out(view, (GdkEventAny *)event);
-
-	  view->last_offset = start_offset;
-
-	  gdk_window_set_cursor(gtk_text_view_get_window(GTK_TEXT_VIEW(text_view),
-							 GTK_TEXT_WINDOW_TEXT),
-				view->hand_cursor);
-
-	  g_signal_emit(G_OBJECT(view),
-			bbs_thread_view_signals[LINK_MOUSE_OVER_SIGNAL],
-			0,
-			event,
-			view->thread,
-			text);
-	}
-
-      G_FREE(text);
-    }
-  else if (text_view_get_iter_at_link_tag_from_event(text_view, &iter,
-						     (GdkEventAny *)event))
-    {
-      int start_offset = gtk_text_iter_get_offset(&iter);
-      if (start_offset != view->last_offset)
-	{
-	  if (view->last_offset != -1)
-	    emit_link_mouse_out(view, (GdkEventAny *)event);
-
-	  view->last_offset = start_offset;
-	  emit_link_mouse_over(view, event);
-	}
-    }
-  else if (view->last_offset != -1)
-    emit_link_mouse_out(view, (GdkEventAny *)event);
-
-  if (GTK_WIDGET_CLASS(parent_class)->motion_notify_event != NULL)
-    result = (*GTK_WIDGET_CLASS(parent_class)->motion_notify_event)(widget,
-								    event);
-
-  return result;
-}
-
-
-static gboolean
-bbs_thread_view_leave_notify(GtkWidget *widget, GdkEventCrossing *event)
-{
-  BBSThreadView *view = BBS_THREAD_VIEW(widget);
-
-  if (view->last_offset != -1)
-    emit_link_mouse_out(view, (GdkEventAny *)event);
-
-  if (GTK_WIDGET_CLASS(parent_class)->leave_notify_event != NULL)
-    return (*GTK_WIDGET_CLASS(parent_class)->leave_notify_event)(widget,
-								 event);
-  return FALSE;
-}
-
-
 static void
-emit_link_mouse_over(BBSThreadView *view, GdkEventMotion *event)
+emit_link_mouse_over(BBSThreadView *view, GdkEventMotion *event,
+		     const gchar *link)
 {
-  gchar *link;
-  if (view->last_offset == -1)
-    return;
-
-  if (view->link_table == NULL)
-    return;
-
-  link = (gchar *)g_hash_table_lookup(view->link_table,
-				      (gconstpointer)view->last_offset);
-
-  if (link == NULL)
-    return;
-
   gdk_window_set_cursor(gtk_text_view_get_window(GTK_TEXT_VIEW(view),
 						 GTK_TEXT_WINDOW_TEXT),
 			view->hand_cursor);
@@ -828,11 +725,6 @@ emit_link_mouse_over(BBSThreadView *view, GdkEventMotion *event)
 static void
 emit_link_mouse_out(BBSThreadView *view, GdkEventAny *event)
 {
-  if (view->last_offset == -1)
-    return;
-
-  view->last_offset = -1;
-
   gdk_window_set_cursor(gtk_text_view_get_window(GTK_TEXT_VIEW(view),
 						 GTK_TEXT_WINDOW_TEXT),
 			view->ibeam_cursor);
@@ -845,42 +737,136 @@ emit_link_mouse_out(BBSThreadView *view, GdkEventAny *event)
 }
 
 
+static gboolean
+bbs_thread_view_motion_notify(GtkWidget *widget, GdkEventMotion *event)
+{
+  GtkTextIter iter;
+  GtkTextView *text_view;
+  BBSThreadView *view;
+
+  g_return_val_if_fail(IS_BBS_THREAD_VIEW(widget)
+		       && event->type == GDK_MOTION_NOTIFY, FALSE);
+
+  view = BBS_THREAD_VIEW(widget);
+  text_view = &view->text_view;
+
+  if (text_view_get_iter_from_event(text_view, &iter, (GdkEventAny *)event))
+    {
+      GtkTextIter start;
+      GtkTextIter end;
+      gchar *text;
+
+      if (gtk_text_buffer_get_selection_bounds(view->text_buffer, &start, &end)
+	  && gtk_text_iter_in_range(&iter, &start, &end)
+	  && (text = gtk_text_iter_get_visible_text(&start, &end)) != NULL)
+	{
+	  int start_offset = gtk_text_iter_get_offset(&iter);
+	  if (start_offset != view->last_offset)
+	    {
+	      if (view->last_offset != -1)
+		emit_link_mouse_out(view, (GdkEventAny *)event);
+
+	      emit_link_mouse_over(view, event, text);
+	      view->last_offset = start_offset;
+	    }
+
+	  g_free(text);
+	}
+      else if (text_view_move_iter_at_link_tag(text_view, &iter))
+	{
+	  int start_offset = gtk_text_iter_get_offset(&iter);
+	  if (start_offset != view->last_offset)
+	    {
+	      const gchar *link;
+
+	      if (view->last_offset != -1)
+		emit_link_mouse_out(view, (GdkEventAny *)event);
+
+	      if (view->link_table != NULL
+		  && (link = g_hash_table_lookup(view->link_table,
+				(gconstpointer)start_offset)) != NULL)
+		{
+		  emit_link_mouse_over(view, event, link);
+		  view->last_offset = start_offset;
+		}
+	      else
+		view->last_offset = -1;
+	    }
+	}
+      else if (view->last_offset != -1)
+	{
+	  emit_link_mouse_out(view, (GdkEventAny *)event);
+	  view->last_offset = -1;
+	}
+    }
+  else if (view->last_offset != -1)
+    {
+      emit_link_mouse_out(view, (GdkEventAny *)event);
+      view->last_offset = -1;
+    }
+
+  if (GTK_WIDGET_CLASS(parent_class)->motion_notify_event != NULL)
+    return (*GTK_WIDGET_CLASS(parent_class)->motion_notify_event)(widget,
+								  event);
+  return FALSE;
+}
+
+
+static gboolean
+bbs_thread_view_leave_notify(GtkWidget *widget, GdkEventCrossing *event)
+{
+  BBSThreadView *view = BBS_THREAD_VIEW(widget);
+
+  if (view->last_offset != -1)
+    {
+      emit_link_mouse_out(view, (GdkEventAny *)event);
+      view->last_offset = -1;
+    }
+
+  if (GTK_WIDGET_CLASS(parent_class)->leave_notify_event != NULL)
+    return (*GTK_WIDGET_CLASS(parent_class)->leave_notify_event)(widget,
+								 event);
+  return FALSE;
+}
+
+
 static gboolean
 bbs_thread_view_button_press(GtkWidget *widget, GdkEventButton *event)
 {
+  gboolean result = FALSE;
+  BBSThreadView *view;
   GtkTextView *text_view;
   GtkTextIter iter;
-  gboolean result = FALSE;
 
   g_return_val_if_fail(IS_BBS_THREAD_VIEW(widget)
 		       && (event->type == GDK_BUTTON_PRESS
 			   || event->type == GDK_2BUTTON_PRESS
 			   || event->type == GDK_3BUTTON_PRESS), FALSE);
-  text_view = GTK_TEXT_VIEW(widget);
+  view = BBS_THREAD_VIEW(widget);
+  text_view = &view->text_view;
 
-  if (text_view_get_iter_at_link_tag_from_event(text_view, &iter,
-						(GdkEventAny *)event))
+  if (text_view_get_iter_from_event(text_view, &iter, (GdkEventAny *)event)
+      && text_view_move_iter_at_link_tag(text_view, &iter))
     {
-      BBSThreadView *view = BBS_THREAD_VIEW(text_view);
-      int start_offset = gtk_text_iter_get_offset(&iter);
-      gchar *link = (view->link_table != NULL)
-	? (gchar *)g_hash_table_lookup(view->link_table,
-				       (gconstpointer)start_offset)
-	: NULL;
-
-      g_signal_emit(G_OBJECT(view),
-		    bbs_thread_view_signals[LINK_MOUSE_PRESS_SIGNAL],
-		    0,
-		    event,
-		    view->thread,
-		    link,
-		    &result);
+      int offset = gtk_text_iter_get_offset(&iter);
+      const gchar *link
+	= (view->link_table != NULL
+	   ? (gchar *)g_hash_table_lookup(view->link_table,
+					  (gconstpointer)offset)
+	   : NULL);
+      if (link != NULL)
+	g_signal_emit(G_OBJECT(view),
+		      bbs_thread_view_signals[LINK_MOUSE_PRESS_SIGNAL],
+		      0,
+		      event,
+		      view->thread,
+		      link,
+		      &result);
     }
 
   if (!result && GTK_WIDGET_CLASS(parent_class)->button_press_event != NULL)
-    result = (*GTK_WIDGET_CLASS(parent_class)->button_press_event)(widget,
-								   event);
-
+    return (*GTK_WIDGET_CLASS(parent_class)->button_press_event)(widget,
+								 event);
   return result;
 }
 
@@ -888,37 +874,39 @@ bbs_thread_view_button_press(GtkWidget *widget, GdkEventButton *event)
 static gboolean
 bbs_thread_view_button_release(GtkWidget *widget, GdkEventButton *event)
 {
+  gboolean result = FALSE;
+  BBSThreadView *view;
   GtkTextView *text_view;
   GtkTextIter iter;
-  gboolean result = FALSE;
 
-  g_return_val_if_fail(GTK_IS_TEXT_VIEW(widget)
+  g_return_val_if_fail(IS_BBS_THREAD_VIEW(widget)
 		       && event->type == GDK_BUTTON_RELEASE, FALSE);
-  text_view = GTK_TEXT_VIEW(widget);
+  view = BBS_THREAD_VIEW(widget);
+  text_view = &view->text_view;
 
-  if (text_view_get_iter_at_link_tag_from_event(text_view, &iter,
-						(GdkEventAny *)event))
+  if (text_view_get_iter_from_event(text_view, &iter, (GdkEventAny *)event)
+      && text_view_move_iter_at_link_tag(text_view, &iter))
     {
-      BBSThreadView *view = BBS_THREAD_VIEW(text_view);
-      int start_offset = gtk_text_iter_get_offset(&iter);
-      gchar *link = (view->link_table != NULL)
-	? (gchar *)g_hash_table_lookup(view->link_table,
-				       (gconstpointer)start_offset)
-	: NULL;
+      int offset = gtk_text_iter_get_offset(&iter);
+      const gchar *link
+	= (view->link_table != NULL
+	   ? (gchar *)g_hash_table_lookup(view->link_table,
+					  (gconstpointer)offset)
+	   : NULL);
 
-      g_signal_emit(G_OBJECT(view),
-		    bbs_thread_view_signals[LINK_MOUSE_RELEASE_SIGNAL],
-		    0,
-		    event,
-		    view->thread,
-		    link,
-		    &result);
+      if (link != NULL)
+	g_signal_emit(G_OBJECT(view),
+		      bbs_thread_view_signals[LINK_MOUSE_RELEASE_SIGNAL],
+		      0,
+		      event,
+		      view->thread,
+		      link,
+		      &result);
     }
 
   if (GTK_WIDGET_CLASS(parent_class)->button_release_event != NULL)
-    result = (*GTK_WIDGET_CLASS(parent_class)->button_release_event)(widget,
-								     event);
-
+    return (*GTK_WIDGET_CLASS(parent_class)->button_release_event)(widget,
+								   event);
   return result;
 }
 
@@ -951,6 +939,20 @@ bbs_thread_view_get_thread(BBSThreadView *view)
 }
 
 
+char *
+bbs_thread_view_get_selected_text(BBSThreadView *view)
+{
+  GtkTextIter start;
+  GtkTextIter end;
+
+  g_return_val_if_fail(IS_BBS_THREAD_VIEW(view), NULL);
+
+  if (!gtk_text_buffer_get_selection_bounds(view->text_buffer, &start, &end))
+    return NULL;	/* 򤵤Ƥʤ */
+  return gtk_text_iter_get_visible_text(&start, &end);
+}
+
+
 void
 bbs_thread_view_clear(BBSThreadView *view)
 {
@@ -1399,11 +1401,11 @@ bbs_thread_view_pop_tags(BBSThreadView *view, GtkTextTag *tag)
 
 
 static void
-bbs_thread_view_scroll_to(BBSThreadView *view, int to_where)
+bbs_thread_view_scroll_to(BBSThreadView *view, BBSThreadViewScrollTo where)
 {
   g_return_if_fail(IS_BBS_THREAD_VIEW(view));
 
-  if (to_where == SCROLL_TO_BUFFER_START)
+  if (where == SCROLL_TO_BUFFER_START)
     bbs_thread_view_scroll_to_start(view);
   else
     bbs_thread_view_scroll_to_end(view);
