@@ -23,7 +23,7 @@
  * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
  * SUCH DAMAGE.
  *
- * $Id: boardlist_view.c,v 1.35 2004/04/07 19:22:21 fuyu Exp $
+ * $Id: boardlist_view.c,v 1.35.2.2 2004/06/11 12:17:14 fuyu Exp $
  */
 
 #include "config.h"
@@ -78,7 +78,7 @@ static void boardlist_view_destroy(GtkObject *object);
 static void boardlist_view_do_on_selection(BoardlistView *view,
 					   int what_to_do);
 static void boardlist_view_do_on_item_at_cursor(BoardlistView *view,
-						int what_to_do);
+						BoardlistViewAction what_to_do);
 
 static void tree_view_row_expanded_cb(GtkTreeView *tree_view,
 				      GtkTreeIter *iter, GtkTreePath *path,
@@ -126,6 +126,30 @@ boardlist_view_get_type(void)
 }
 
 
+GType
+boardlist_view_action_get_type(void)
+{
+  static GType action_type = 0;
+  if (action_type == 0)
+    {
+      static const GEnumValue values[] =
+	{
+	  { TOGGLE_HIDDEN_STATE,
+	    (char *)"TOGGLE_HIDDEN_STATE", (char *)"toggle-hidden-state" },
+	  { KILL_ITEM,
+	    (char *)"KILL_ITEM", (char *)"kill-item" },
+	  { ADVANCE_VIEW,
+	    (char *)"ADVANCE_VIEW", (char *)"advance-view" },
+	  { BACK_VIEW,
+	    (char *)"BACK_VIEW", (char *)"go-back-view" },
+	  { 0, NULL, NULL },
+	};
+      action_type = g_enum_register_static("BoardlistViewAction", values);
+    }
+  return action_type;
+}
+
+
 typedef struct _CategoryNodeInfo
 {
   GtkTreeRowReference *reference;
@@ -158,14 +182,6 @@ enum {
 };
 
 
-enum {
-  TOGGLE_HIDDEN_STATE,
-  KILL_ITEM,
-  ADVANCE_VIEW,
-  BACK_VIEW,
-};
-
-
 enum {
   POPUP_MENU_OPEN_SELECTION = 1,
   POPUP_MENU_OPEN_SELECTION_IN_TAB,
@@ -655,9 +671,9 @@ boardlist_view_class_init(BoardlistViewClass *klass)
 		 G_SIGNAL_RUN_LAST | G_SIGNAL_ACTION,
 		 G_STRUCT_OFFSET(BoardlistViewClass, do_on_item_at_cursor),
 		 NULL, NULL,
-		 ochusha_marshal_VOID__INT,
+		 ochusha_marshal_VOID__ENUM,
 		 G_TYPE_NONE, 1,
-		 G_TYPE_INT);
+		 BOARDLIST_VIEW_ACTION_TYPE);
 
   boardlist_view_signals[URI_DROPPED_SIGNAL] =
     g_signal_new("uri_dropped",
@@ -1394,7 +1410,8 @@ boardlist_view_do_on_selection(BoardlistView *view, int what_to_do)
 
 
 static void
-boardlist_view_do_on_item_at_cursor(BoardlistView *view, int what_to_do)
+boardlist_view_do_on_item_at_cursor(BoardlistView *view,
+				    BoardlistViewAction what_to_do)
 {
   GtkTreeIter iter;
   GtkTreePath *path = NULL;
