@@ -23,7 +23,7 @@
  * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
  * SUCH DAMAGE.
  *
- * $Id: download_ui.c,v 1.11.2.2 2004/05/15 15:39:48 fuyu Exp $
+ * $Id: download_ui.c,v 1.11.2.3 2004/06/04 11:59:21 fuyu Exp $
  */
 
 #include "config.h"
@@ -106,12 +106,11 @@ access_progressed_cb(OchushaAsyncBuffer *buffer,
       else
 	{
 	  char message[256];
-	  int percent = (bytes_read / bytes_total) * 100;
+	  double frac = (double)bytes_read / (double)bytes_total;
 	  snprintf(message, 256, "%d%% (%d/%d)",
-		   percent, bytes_read, bytes_total);
+		   (int)(frac * 100.0), bytes_read, bytes_total);
 	  gtk_progress_bar_set_text(pbar, message);
-	  gtk_progress_bar_set_fraction(pbar,
-					(double)bytes_read / (double)bytes_total);
+	  gtk_progress_bar_set_fraction(pbar, frac);
 	}
     }
 
