@@ -23,7 +23,7 @@
  * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
  * SUCH DAMAGE.
  *
- * $Id: ochusha_board_jbbs.c,v 1.17 2004/02/05 01:04:55 fuyu Exp $
+ * $Id: ochusha_board_jbbs.c,v 1.17.2.1 2004/05/24 10:00:55 fuyu Exp $
  */
 
 #include "config.h"
@@ -660,7 +660,7 @@ collect_dropped_thread(gpointer data, gpointer user_data)
 	    {
 	      thread->flags |= OCHUSHA_BBS_THREAD_DAT_DROPPED;
 	      thread->number_of_responses_on_server = 0;
-	      args->thread_list = g_slist_append(args->thread_list, thread);
+	      args->thread_list = g_slist_prepend(args->thread_list, thread);
 	      if (args->each_thread_cb != NULL)
 		if (!(*args->each_thread_cb)(thread, args->callback_data))
 		  args->each_thread_cb = FALSE;
@@ -672,7 +672,7 @@ collect_dropped_thread(gpointer data, gpointer user_data)
 	}
       else
 	thread->board->dropped_list
-	  = g_slist_append(thread->board->dropped_list, thread);
+	  = g_slist_prepend(thread->board->dropped_list, thread);
     }
   else
     OCHU_OBJECT_UNREF(data);	/* Ƥ륹thread_list
@@ -862,7 +862,7 @@ refresh_threadlist_by_subject_txt(OchushaBulletinBoard *board,
 		thread->number_of_responses_on_server = n_responses_on_server;
 
 		OCHU_OBJECT_REF(thread);
-		thread_list = g_slist_append(thread_list, thread);
+		thread_list = g_slist_prepend(thread_list, thread);
 
 		if (each_thread_cb != NULL
 		    && !(*each_thread_cb)(thread, callback_data))
@@ -1145,7 +1145,7 @@ refresh_threadlist_by_subback_html(OchushaBulletinBoard *board,
 		thread->number_of_responses_on_server = n_responses_on_server;
 
 		OCHU_OBJECT_REF(thread);
-		thread_list = g_slist_append(thread_list, thread);
+		thread_list = g_slist_prepend(thread_list, thread);
 
 		if (each_thread_cb != NULL
 		    && !(*each_thread_cb)(thread, callback_data))
