@@ -23,7 +23,7 @@
  * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
  * SUCH DAMAGE.
  *
- * $Id: ochusha_network_broker.c,v 1.60.2.2 2004/05/04 03:05:09 fuyu Exp $
+ * $Id: ochusha_network_broker.c,v 1.60.2.5 2004/05/24 10:02:27 fuyu Exp $
  */
 
 #include "config.h"
@@ -2089,21 +2089,19 @@ ochusha_network_broker_read_from_url_full(OchushaNetworkBroker *broker,
       if (buffer != NULL && buffer->length > 0)
 	return buffer;
 
-      if (mode == OCHUSHA_NETWORK_BROKER_IMAGE_CACHE)
+      fd = ochusha_config_image_cache_open_file(broker->config,
+						cache_url, O_RDONLY);
+      if (fd < 0)
 	{
-	  fd = ochusha_config_image_cache_open_file(broker->config,
-						    cache_url, O_RDONLY);
+	  char *cache_name = ochusha_utils_url_encode_string(cache_url);
+	  fd = ochusha_config_open_file(broker->config, cache_name,
+					"image_cache", O_RDONLY);
+	  G_FREE(cache_name);
+
 	  if (fd < 0)
-	    {
-	      char *cache_name = ochusha_utils_url_encode_string(cache_url);
-	      fd = ochusha_config_open_file(broker->config, cache_name,
-					    "image_cache", O_RDONLY);
-	      G_FREE(cache_name);
-	    }
+	    fd = ochusha_config_cache_open_file(broker->config,
+						cache_url, O_RDONLY);
 	}
-      else
-	fd = ochusha_config_cache_open_file(broker->config,
-					    cache_url, O_RDONLY);
 	
       if (fd >= 0)
 	{
@@ -2128,15 +2126,14 @@ ochusha_network_broker_read_from_url_full(OchushaNetworkBroker *broker,
 	  OCHU_OBJECT_REF(broker);
 	  OCHU_OBJECT_REF(buffer);
 
-	  if (modestly)
-	    commit_modest_job(job);
-	  else
-	    commit_job(job);
+	  commit_job(job);
 
 	  return buffer;
 	}
 
-      return buffer;
+      if (mode != OCHUSHA_NETWORK_BROKER_CACHE_AS_IS
+	  || broker->config->offline)
+	return buffer;
     }
 
   switch (mode)
@@ -2365,13 +2362,23 @@ ochusha_network_broker_try_post_raw(OchushaNetworkBroker *broker,
 
   ghttp_set_body(request, (char *)body, strlen(body));
   ghttp_prepare(request);
+#if 1
   ghttp_load_verify_locations(request, OCHUSHA_CA_BUNDLE_CRT, NULL);
+#else
+  ghttp_load_verify_locations(request, NULL, NULL);
+#endif
 
   if (ghttp_process(request) == ghttp_error)
     {
-      ochusha_network_broker_output_log(broker, _("Posting failed: ghttp_process() returns ghttp_error\n"));
 #if DEBUG_POST_MOST
       fprintf(stderr, "ghttp_process() returns ghttp_error\n");
+#endif
+#if DEBUG_NETWORK || DEBUG_POST
+      snprintf(message, LOG_MESSAGE_SIZE, _("Posting failed: %s\n"),
+	       ghttp_get_error(request));
+      ochusha_network_broker_output_log(broker, message);
+#else
+      ochusha_network_broker_output_log(broker, _("Posting failed: ghttp_process() returns ghttp_error\n"));
 #endif
       ghttp_request_destroy(request);
       if (status != NULL)
